@@ -1,21 +1,15 @@
-import os
-import random
-import tempfile
-from copy import copy
-from functools import partial
-from multiprocessing import cpu_count
-import pickle
 import json
-from glob import glob
 import logging
+import os
+from functools import partial
+from glob import glob
 
 import numpy as np
 import tensorflow as tf
 from joblib import Parallel, delayed
-import pandas as pd
 
-from .special_tokens import BOS_TOKEN, EOS_TOKEN, TRAIN, EVAL
-from .bert_preprocessing.create_bert_features import create_bert_features
+from .bert_preprocessing.create_bert_features import create_bert_features, create_multimodal_bert_features
+from .special_tokens import BOS_TOKEN, EOS_TOKEN, EVAL, TRAIN
 
 
 def _float_list_feature(value):
@@ -60,8 +54,14 @@ def serialize_fn(features: dict, return_feature_desc=False):
 
             feature_desc['{}_shape'.format(
                 feature_name)] = 'int64'
-            feature_desc['{}_shape_value'.format(feature_name)] = [
-                None for _ in feature.shape]
+
+            # this seems not a good idea
+            if len(feature.shape) > 1:
+                feature_desc['{}_shape_value'.format(feature_name)] = [
+                    None] + list(feature.shape[1:])
+            else:
+                feature_desc['{}_shape_value'.format(feature_name)] = [
+                    None for _ in feature.shape]
 
         elif np.issubdtype(type(feature), np.float):
             features_tuple[feature_name] = _float_feature(feature)
@@ -102,17 +102,14 @@ def serialize_fn(features: dict, return_feature_desc=False):
 def make_tfrecord(data_list, output_dir, serialize_fn, mode='train', shards_per_file=10000, prefix='', **kwargs):
 
     # create output tfrecord path
-    file_list = [os.path.join(
-        output_dir, prefix, '{}_{:05d}.tfrecord'.format(mode, i)) for i in range(len(data_list))]
-
-    os.makedirs(os.path.dirname(file_list[0]), exist_ok=True)
+    os.makedirs(os.path.join(
+        output_dir, prefix), exist_ok=True)
 
     def _write_fn(d_list, path, serialize_fn, mode='train'):
         logging.info('Writing {}'.format(path))
         feature_desc_path = os.path.join(os.path.dirname(
             path), '{}_feature_desc.json'.format(mode))
 
-        return_list = []
         with tf.io.TFRecordWriter(path) as writer:
             for features in d_list:
                 example, feature_desc = serialize_fn(
@@ -124,8 +121,22 @@ def make_tfrecord(data_list, output_dir, serialize_fn, mode='train', shards_per_
 
     _write_part_fn = partial(_write_fn, serialize_fn=serialize_fn, mode=mode)
 
-    for x, y in zip(data_list, file_list):
-        _write_part_fn(d_list=x, path=y)
+    x = []
+    shard_count = 0
+    for idx, example in enumerate(data_list):
+        x.append(example)
+        if idx % shards_per_file == 0 and idx:
+            path = os.path.join(
+                output_dir, prefix, '{}_{:05d}.tfrecord'.format(mode, shard_count))
+            shard_count += 1
+            _write_part_fn(d_list=x, path=path)
+            x = []
+
+    # add remaining
+    if x:
+        path = os.path.join(
+            output_dir, prefix, '{}_{:05d}.tfrecord'.format(mode, shard_count))
+        _write_part_fn(d_list=x, path=path)
 
 
 def write_single_problem_chunk_tfrecord(problem,
@@ -153,15 +164,25 @@ def write_single_problem_chunk_tfrecord(problem,
         for i in range(0, len(example_list), 10000):
             data_shards.append(example_list[i:i + 10000])
 
-        part_fn = partial(create_bert_features, problem=problem,
-                          label_encoder=label_encoder,
-                          params=params,
-                          tokenizer=tokenizer,
-                          mode=mode,
-                          problem_type=problem_type,
-                          is_seq=is_seq)
-        data_list = Parallel(min(cpu_count(), len(data_shards)))(delayed(part_fn)(example_list=d_list)
-                                                                 for d_list in data_shards)
+        if isinstance(inputs_list[0], dict) and 'a' not in inputs_list[0]:
+            part_fn = partial(create_multimodal_bert_features, problem=problem,
+                              label_encoder=label_encoder,
+                              params=params,
+                              tokenizer=tokenizer,
+                              mode=mode,
+                              problem_type=problem_type,
+                              is_seq=is_seq)
+        else:
+            part_fn = partial(create_bert_features, problem=problem,
+                              label_encoder=label_encoder,
+                              params=params,
+                              tokenizer=tokenizer,
+                              mode=mode,
+                              problem_type=problem_type,
+                              is_seq=is_seq)
+        data_list = Parallel(min(params.num_cpus, len(data_shards)))(delayed(part_fn)(example_list=d_list)
+                                                                     for d_list in data_shards)
+
         return data_list
 
     # single problem in problem_chunk
@@ -170,6 +191,8 @@ def write_single_problem_chunk_tfrecord(problem,
             problem = problem[0]
         data_list = _make_single_problem_data_list(
             problem, inputs_list=inputs_list, target_list=target_list, label_encoder=label_encoder)
+        data_list = [
+            item for sublist in data_list for item in sublist]
 
     # multiple problem in problem chunk
     else:
@@ -181,9 +204,13 @@ def write_single_problem_chunk_tfrecord(problem,
         for pro in problem:
             data_shards = _make_single_problem_data_list(
                 pro, inputs_list=inputs_list[pro], target_list=target_list[pro], label_encoder=label_encoder[pro])
+
             data_dict[pro] = [
                 item for sublist in data_shards for item in sublist]
-            column_list.append(list(data_dict[pro][0].keys()))
+            try:
+                column_list.append(list(data_dict[pro][0].keys()))
+            except IndexError:
+                raise IndexError("Problem {} has no data".format(pro))
 
         # get intersection and use as ensure features are the same
         join_key = list(set(column_list[0]).intersection(*column_list[1:]))
@@ -201,10 +228,91 @@ def write_single_problem_chunk_tfrecord(problem,
                         )
                     d.update(data_dict[pro].pop(0))
             flat_data_list.append(d)
+        data_list = flat_data_list
 
-        data_list = []
-        for i in range(0, len(flat_data_list), 10000):
-            data_list.append(flat_data_list[i:i + 10000])
+    if isinstance(problem, list):
+        problem = '_'.join(sorted(problem))
+
+    make_tfrecord(data_list=data_list,
+                  output_dir=params.tmp_file_dir,
+                  serialize_fn=serialize_fn,
+                  prefix=problem,
+                  mode=mode)
+
+
+def write_single_problem_gen_tfrecord(problem,
+                                      gen,
+                                      label_encoder,
+                                      params,
+                                      tokenizer,
+                                      mode):
+
+    def _make_single_problem_data_gen(problem, example_list, label_encoder):
+        problem_type = params.problem_type[problem]
+
+        # whether this problem is sequential labeling
+        # for sequential labeling, targets needs to align with any
+        # change of inputs
+        is_seq = problem_type in ['seq_tag']
+        first_example = next(gen)
+
+        if isinstance(first_example[0], dict) and 'a' not in first_example[0]:
+            part_fn = partial(create_multimodal_bert_features, problem=problem,
+                              label_encoder=label_encoder,
+                              params=params,
+                              tokenizer=tokenizer,
+                              mode=mode,
+                              problem_type=problem_type,
+                              is_seq=is_seq)
+        else:
+            part_fn = partial(create_bert_features, problem=problem,
+                              label_encoder=label_encoder,
+                              params=params,
+                              tokenizer=tokenizer,
+                              mode=mode,
+                              problem_type=problem_type,
+                              is_seq=is_seq)
+
+        # get num_cpus*per_cpu_buffer instances
+        cpus = params.num_cpus
+        tmp_instances_list = [[] for _ in range(cpus)]
+        per_cpu_buffer = params.per_cpu_buffer
+        for example in example_list:
+            is_full = False
+            for i in range(cpus):
+                if len(tmp_instances_list[i]) < per_cpu_buffer:
+                    tmp_instances_list[i].append(example)
+                    if i == cpus - 1 and len(tmp_instances_list[i]) == per_cpu_buffer:
+                        is_full = True
+                    break
+
+            # if is full, process and clear
+            if is_full:
+                data_list = Parallel(min(params.num_cpus, len(tmp_instances_list)))(delayed(part_fn)(example_list=d_list)
+                                                                                    for d_list in tmp_instances_list)
+                for d_list in data_list:
+                    for d in d_list:
+                        yield d
+
+                tmp_instances_list = [[] for _ in range(cpus)]
+        # process remaining
+        for tmp_list in tmp_instances_list:
+            if tmp_list:
+                d_list = part_fn(example_list=tmp_list)
+                for d in d_list:
+                    yield d
+
+    # single problem in problem_chunk
+    if isinstance(problem, str) or (isinstance(problem, list) and len(problem) == 1):
+        if isinstance(problem, list):
+            problem = problem[0]
+        data_list = _make_single_problem_data_gen(
+            problem, example_list=gen, label_encoder=label_encoder)
+
+    # multiple problem in problem chunk
+    else:
+        raise NotImplementedError(
+            'The problem returns generator, which dose not support & chaining')
 
     if isinstance(problem, list):
         problem = '_'.join(sorted(problem))
@@ -263,15 +371,15 @@ def write_tfrecord(params, replace=False):
                         target_list_dict[p] = res_dict['target_list']
                         label_encoder_dict[p] = res_dict['label_encoder']
 
-                        write_single_problem_chunk_tfrecord(
-                            problem=problem_list,
-                            inputs_list=input_list_dict,
-                            target_list=target_list_dict,
-                            label_encoder=label_encoder_dict,
-                            params=params,
-                            tokenizer=tokenizer,
-                            mode=mode
-                        )
+                    write_single_problem_chunk_tfrecord(
+                        problem=problem_list,
+                        inputs_list=input_list_dict,
+                        target_list=target_list_dict,
+                        label_encoder=label_encoder_dict,
+                        params=params,
+                        tokenizer=tokenizer,
+                        mode=mode
+                    )
 
 
 def make_feature_desc(feature_desc_dict: dict):
@@ -351,17 +459,19 @@ def get_dummy_features(dataset_dict, feature_desc_dict):
         dummy_features -- dict of dummy tensors
     """
 
-    feature_keys = [list(d.output_shapes.keys())
+    feature_keys = [list(d.element_spec.keys())
                     for _, d in dataset_dict.items()]
     common_features_accross_problems = set(
         feature_keys[0]).intersection(*feature_keys[1:])
 
     dummy_features = {}
     for problem, problem_dataset in dataset_dict.items():
-        output_types = problem_dataset.output_types
-        dummy_features.update({k: tf.cast(tf.constant(shape=[1 for _ in feature_desc_dict.get('{}_shape_value'.format(k), [])], value=0), v)
+        output_types = {k: v.dtype for k,
+                        v in problem_dataset.element_spec.items()}
+        dummy_features.update({k: tf.cast(tf.constant(shape=[1 if s is None else s for s in feature_desc_dict.get('{}_shape_value'.format(k), [])], value=0), v)
                                for k, v in output_types.items()
                                if k not in common_features_accross_problems})
+
     return dummy_features
 
 
@@ -407,13 +517,18 @@ def read_tfrecord(params, mode: str):
             open(os.path.join(file_dir, '{}_feature_desc.json'.format(mode))))
         all_feature_desc_dict.update(feature_desc_dict)
         feature_desc = make_feature_desc(feature_desc_dict)
-        dataset = tf.data.TFRecordDataset(tfrecord_path_list)
+        dataset = tf.data.TFRecordDataset(
+            tfrecord_path_list, num_parallel_reads=tf.data.experimental.AUTOTUNE)
         dataset = dataset.map(lambda x: tf.io.parse_single_example(
-            x, feature_desc)).map(reshape_tensors_in_dataset).map(
-                lambda x: set_shape_for_dataset(x, feature_desc_dict)
+            serialized=x, features=feature_desc), num_parallel_calls=tf.data.experimental.AUTOTUNE).map(
+                reshape_tensors_in_dataset, num_parallel_calls=tf.data.experimental.AUTOTUNE)
+        dataset = dataset.map(
+            lambda x: set_shape_for_dataset(x, feature_desc_dict),
+            num_parallel_calls=tf.data.experimental.AUTOTUNE
         )
         for p in problem_list:
-            dataset = dataset.map(lambda x: add_loss_multiplier(x, p))
+            dataset = dataset.map(lambda x: add_loss_multiplier(x, p),
+                                  num_parallel_calls=tf.data.experimental.AUTOTUNE)
         dataset_dict[problem] = dataset
 
     # add dummy features
@@ -421,6 +536,7 @@ def read_tfrecord(params, mode: str):
     for problem_list in params.problem_chunk:
         problem = '_'.join(sorted(problem_list))
         dataset_dict[problem] = dataset_dict[problem].map(
-            lambda x: add_dummy_features_to_dataset(x, dummy_features)
+            lambda x: add_dummy_features_to_dataset(x, dummy_features),
+            num_parallel_calls=tf.data.experimental.AUTOTUNE
         ).repeat()
     return dataset_dict
